@@ -21,6 +21,7 @@ from datetime import datetime
 from argparse import ArgumentParser, Action, SUPPRESS, RawDescriptionHelpFormatter
 from contextlib import ExitStack
 import importlib_metadata
+from ruamel.yaml import YAML
 from . import *
 from ._util import (
     provision_run_dir,
@@ -39,22 +40,7 @@ quant_warning = False
 def main(args=None):
     sys.setrecursionlimit(1_000_000)  # permit as much call stack depth as OS can give us
 
-    parser = ArgumentParser("miniwdl")
-    parser.add_argument(
-        "--version",
-        nargs=0,
-        action=PipVersionAction,
-        help="show miniwdl package version information",
-    )
-    subparsers = parser.add_subparsers()
-    subparsers.required = True
-    subparsers.dest = "command"
-    fill_common(fill_check_subparser(subparsers))
-    fill_common(fill_cromwell_subparser(subparsers), path=False)  # FIXME path issue #131
-    fill_common(fill_run_subparser(subparsers))
-    fill_common(fill_run_self_test_subparser(subparsers))
-    fill_common(fill_localize_subparser(subparsers))
-
+    parser = create_arg_parser()
     argcomplete.autocomplete(parser)
 
     replace_COLUMNS = os.environ.get("COLUMNS", None)
@@ -97,6 +83,25 @@ def main(args=None):
     sys.exit(0)
 
 
+def create_arg_parser():
+    parser = ArgumentParser("miniwdl")
+    parser.add_argument(
+        "--version",
+        nargs=0,
+        action=PipVersionAction,
+        help="show miniwdl package version information",
+    )
+    subparsers = parser.add_subparsers()
+    subparsers.required = True
+    subparsers.dest = "command"
+    fill_common(fill_check_subparser(subparsers))
+    fill_common(fill_cromwell_subparser(subparsers), path=False)  # FIXME path issue #131
+    fill_common(fill_run_subparser(subparsers))
+    fill_common(fill_run_self_test_subparser(subparsers))
+    fill_common(fill_localize_subparser(subparsers))
+    return parser
+
+
 class PipVersionAction(Action):
     def __call__(self, parser, namespace, values, option_string=None):
         try:
@@ -135,7 +140,8 @@ def fill_common(subparser, path=True):
             action="append",
             help="local directory to search for imports",
         )
-    subparser.add_argument(
+    group = subparser.add_argument_group("debugging")
+    group.add_argument(
         "--debug", action="store_true", help="maximally verbose logging & exception tracebacks"
     )
 
@@ -152,7 +158,7 @@ def fill_check_subparser(subparsers):
         formatter_class=RawDescriptionHelpFormatter,
     )
     check_parser.add_argument(
-        "uri", metavar="URI", type=str, nargs="+", help="WDL document filename/URI"
+        "uri", metavar="WDL_URI", type=str, nargs="+", help="WDL document filename/URI"
     )
     check_parser.add_argument(
         "--strict",
@@ -356,7 +362,7 @@ def fill_run_subparser(subparsers):
         "--input",
         metavar="INPUT.json",
         dest="input_file",
-        help="Cromwell-style input JSON file; command-line inputs will be merged in",
+        help="Cromwell-style input JSON object, filename, or -; command-line inputs will be merged in",
     )
     group.add_argument(
         "--empty",
@@ -410,28 +416,9 @@ def fill_run_subparser(subparsers):
         default=None,
         help="configuration file to load (in preference to file named by MINIWDL_CFG environment, or XDG_CONFIG_{HOME,DIRS}/miniwdl.cfg)",
     )
-    group.add_argument(
-        "-@",
-        metavar="N",
-        dest="max_tasks",
-        type=int,
-        default=None,
-        help="maximum concurrent tasks (default: # host processors, effectively lower when tasks require multiple processors)",
-    )
-    group.add_argument(
-        "--runtime-cpu-max",
-        metavar="N",
-        type=int,
-        default=None,
-        help="maximum effective runtime.cpu for any task (default: # host processors)",
-    )
-    group.add_argument(
-        "--runtime-memory-max",
-        metavar="N",
-        type=str,
-        default=None,
-        help="maximum effective runtime.memory for any task (default: total host memory)",
-    )
+    group.add_argument("-@", metavar="N", dest="max_tasks", type=int, default=None, help=SUPPRESS)
+    group.add_argument("--runtime-cpu-max", metavar="N", type=int, default=None, help=SUPPRESS)
+    group.add_argument("--runtime-memory-max", metavar="N", type=str, default=None, help=SUPPRESS)
     group.add_argument(
         "--runtime-defaults",
         metavar="JSON",
@@ -596,7 +583,7 @@ def runner(
             sys.exit(0)
 
         # debug logging
-        versionlog = {}
+        versionlog = {"python": sys.version}
         for pkg in ["miniwdl", "docker", "lark-parser", "argcomplete", "pygtail"]:
             try:
                 versionlog[pkg] = str(importlib_metadata.version(pkg))
@@ -841,17 +828,23 @@ def runner_input_json_file(available_inputs, namespace, input_file, root):
     if input_file:
         input_file = input_file.strip()
     if input_file:
+        input_json = None
         if input_file[0] == "{":
-            input_json = json.loads(input_file)
+            input_json = input_file
         elif input_file == "-":
-            input_json = json.load(sys.stdin)
+            input_json = sys.stdin.read()
         else:
-            input_json = json.loads(
+            input_json = (
                 asyncio.get_event_loop()
                 .run_until_complete(read_source(input_file, [], None))
                 .source_text
             )
-        ans = values_from_json(input_json, available_inputs, namespace=namespace)
+        input_json = YAML(typ="safe", pure=True).load(input_json)
+        try:
+            ans = values_from_json(input_json, available_inputs, namespace=namespace)
+        except Error.InputError as exn:
+            die("check JSON input; " + str(exn))
+            sys.exit(2)
 
         # join relative file paths to the cwd
 
