@@ -2,11 +2,12 @@
 `miniwdl <https://github.com/chanzuckerberg/miniwdl/>`_ is a developer toolkit and local runner for
 the bioinformatics-focused `Workflow Description Language (WDL) <http://openwdl.org/>`_. This
 documentation covers the Python3 ``WDL`` package facilitating parsing & static analysis of WDL
-documents.
+documents. Simply ``import WDL`` once miniwdl has been installed.
 
 * `GitHub repo <https://github.com/chanzuckerberg/miniwdl/>`_ for installation and further background
 * `Codelabs <https://miniwdl.readthedocs.io/en/latest/WDL.html#python-codelabs>`_ on using this package
 """
+import sys
 import os
 import errno
 import inspect
@@ -128,6 +129,10 @@ async def resolve_file_import(uri: str, path: List[str], importer: Optional[Docu
     relative ``uri`` is resolved by first joining it to either, the directory of the importer
     document (if any), or the process current working directory (otherwise). Failing that, it's
     searched in the ``path`` directories (in reverse order).
+
+    Security-focused applications may wish to override ``read_source`` with logic to restrict
+    allowable results of ``resolve_file_import``, to prevent WDL source code from triggering access
+    to arbitrary filesystem paths. No such restrictions are applied by default.
     """
     return await Tree.resolve_file_import(uri, path, importer)
 
@@ -232,7 +237,12 @@ def values_from_json(
             if isinstance(ty, Tree.Decl):
                 ty = ty.type
             assert isinstance(ty, Type.Base)
-            ans = ans.bind(key2, Value.from_json(ty, values_json[key]))
+            try:
+                ans = ans.bind(key2, Value.from_json(ty, values_json[key]))
+            except Error.InputError as exn:
+                raise Error.InputError(exn.args[0] + f" (in {key})").with_traceback(
+                    sys.exc_info()[2]
+                )
     if required:
         missing = required.subtract(ans)
         if missing:
