@@ -513,32 +513,8 @@ def TerminationSignalFlag(logger: logging.Logger) -> Iterator[Callable[[], bool]
                 _terminating = None
 
 
-@export
-class CustomDeepCopyMixin:
-    """
-    Mixin class overrides __deepcopy__ to consult an internal list of attribute names to be merely
-    shallow-copied when the time comes. Useful for attributes referencing large, immutable data
-    structures.
-
-    Override class variable _shallow_copy_attrs to a list of the attribute names to be
-    shallow-copied.
-    """
-
-    _shallow_copy_attrs: Optional[List[str]] = None
-
-    def __deepcopy__(self, memo: Dict[int, Any]) -> Any:  # pyre-ignore
-        cls = self.__class__
-        cp = cls.__new__(cls)
-        memo[id(self)] = cp
-        for k in self._shallow_copy_attrs or []:
-            v = self.__dict__[k]
-            memo[id(v)] = v
-        for k, v in self.__dict__.items():
-            setattr(cp, k, copy.deepcopy(v, memo))
-        return cp
-
-
 byte_size_units = {
+    "B": 1,
     "K": 1000,
     "KB": 1000,
     "Ki": 1024,
