@@ -496,6 +496,8 @@ class SwarmContainer(TaskContainer):
                 )
 
                 # poll for container exit
+                running_states = {"preparing", "running"}
+                was_running = False
                 while exit_code is None:
                     time.sleep(random.uniform(1.0, 2.0))  # spread out work over the GIL
                     if terminating():
@@ -506,12 +508,14 @@ class SwarmContainer(TaskContainer):
                         if not quiet:
                             self.poll_service(logger, svc, verbose=True)
                         raise Terminated(quiet=quiet)
-                    was_running = "running" in self._observed_states
                     exit_code = self.poll_service(logger, svc)
+                    if not was_running and self._observed_states.intersection(running_states):
+                        # indicate actual container start in status bar
+                        # 'preparing' is when docker is pulling and extracting the image, which can
+                        # be a lengthy and somewhat intensive operation, so we count it as running.
+                        cleanup.enter_context(_statusbar.task_running(cpu, memory_reservation))
+                        was_running = True
                     if "running" in self._observed_states:
-                        if not was_running:
-                            # indicate actual container start in status bar
-                            cleanup.enter_context(_statusbar.task_running(cpu, memory_reservation))
                         poll_stderr()
 
                 logger.debug(
@@ -629,14 +633,11 @@ class SwarmContainer(TaskContainer):
         svc.reload()
         assert svc.attrs["Spec"]["Labels"]["miniwdl_run_id"] == self.run_id
         tasks = svc.tasks()
-        # logger.debug(_("xxx", service=svc.attrs, tasks=tasks))
         if tasks:
             assert len(tasks) == 1, "docker service should have at most 1 task"
             status = tasks[0]["Status"]
             status["DesiredState"] = tasks[0].get("DesiredState", None)
-            if verbose:
-                logger.info(_("docker task status", **status))
-            elif logger.isEnabledFor(logging.DEBUG):
+            if logger.isEnabledFor(logging.DEBUG):
                 logger.debug(_("docker task status", **status))
         else:
             assert (
@@ -706,8 +707,9 @@ class SwarmContainer(TaskContainer):
         alternatives and their problems.
         """
         if not self.cfg["task_runtime"].get_bool("as_user") and (os.geteuid() or os.getegid()):
+            paste = shlex.quote(os.path.join(self.container_dir, "work"))
             script = f"""
-            chown -RP {os.geteuid()}:{os.getegid()} {shlex.quote(os.path.join(self.container_dir, 'work'))}
+            (find {paste} -type d && find {paste} -type f) | xargs -P 10 chown -P {os.geteuid()}:{os.getegid()}
             """.strip()
             volumes = {self.host_dir: {"bind": self.container_dir, "mode": "rw"}}
             logger.debug(_("post-task chown", script=script, volumes=volumes))
@@ -717,7 +719,7 @@ class SwarmContainer(TaskContainer):
                     chowner = client.containers.run(
                         "alpine:3",
                         name=self.unique_service_name("chown-" + self.run_id),
-                        command=["/bin/ash", "-c", script],
+                        command=["/bin/ash", "-eo", "pipefail", "-c", script],
                         volumes=volumes,
                         detach=True,
                     )
@@ -981,16 +983,10 @@ def _eval_task_inputs(
     container.add_files(_filenames(posix_inputs))
 
     # copy posix_inputs with all Files mapped to their in-container paths
-    def map_files(v: Value.Base) -> Value.Base:
-        if isinstance(v, Value.File):
-            v.value = container.input_file_map[v.value]
-        for ch in v.children:
-            map_files(ch)
-        return v
+    def map_files(fn: str) -> str:
+        return container.input_file_map[fn]
 
-    container_inputs = posix_inputs.map(
-        lambda binding: Env.Binding(binding.name, map_files(copy.deepcopy(binding.value)))
-    )
+    container_inputs = Value.rewrite_env_files(posix_inputs, map_files)
 
     # initialize value environment with the inputs
     container_env = Env.Bindings()
@@ -1214,25 +1210,22 @@ def _eval_task_outputs(
 ) -> Env.Bindings[Value.Base]:
 
     # helper to rewrite Files from in-container paths to host paths
-    def rewrite_files(v: Value.Base, output_name: str) -> None:
-        if isinstance(v, Value.File):
-            host_file = container.host_file(v.value)
-            if host_file is None:
-                logger.warning(
-                    _(
-                        "output file not found in container (error unless declared type is optional)",
-                        name=output_name,
-                        file=v.value,
-                    )
+    def rewriter(fn: str, output_name: str) -> str:
+        host_file = container.host_file(fn)
+        if host_file is None:
+            logger.warning(
+                _(
+                    "output file not found in container (error unless declared type is optional)",
+                    name=output_name,
+                    file=fn,
                 )
-            else:
-                logger.debug(_("output file", container=v.value, host=host_file))
-            # We may overwrite File.value with None, which is an invalid state, then we'll fix it
-            # up (or abort) below. This trickery is because we don't, at this point, know whether
-            # the 'desired' output type is File or File?.
-            v.value = host_file
-        for ch in v.children:
-            rewrite_files(ch, output_name)
+            )
+        else:
+            logger.debug(_("output file", container=fn, host=host_file))
+        # We may overwrite File.value with None, which is an invalid state, then we'll fix it
+        # up (or abort) below. This trickery is because we don't, at this point, know whether
+        # the 'desired' output type is File or File?.
+        return host_file  # pyre-fixme
 
     stdlib = OutputStdLib(logger, container)
     outputs = Env.Bindings()
@@ -1256,11 +1249,10 @@ def _eval_task_outputs(
         # compound values)
 
         # First bind the value as-is in the environment, so that subsequent output expressions will
-        # "see" the in-container path(s) if they use this binding. (Copy it though, because we'll
-        # then clobber v)
-        env = env.bind(decl.name, copy.deepcopy(v))
+        # "see" the in-container path(s) if they use this binding.
+        env = env.bind(decl.name, v)
         # Rewrite each File.value to either a host path, or None if the file doesn't exist.
-        rewrite_files(v, decl.name)
+        v = Value.rewrite_files(v, lambda fn: rewriter(fn, decl.name))
         # File.coerce has a special behavior for us so that, if the value is None:
         #   - produces Value.Null() if the desired type is File?
         #   - raises FileNotFoundError otherwise.
