@@ -52,13 +52,12 @@ from .._util import (
     write_atomic,
     write_values_json,
     provision_run_dir,
-    LOGGING_FORMAT,
     TerminationSignalFlag,
     LoggingFileHandler,
     compose_coroutines,
 )
 from .._util import StructuredLogMessage as _
-from . import config
+from . import config, _statusbar
 from .cache import CallCache
 from .error import RunFailed, Terminated, error_json
 
@@ -194,8 +193,6 @@ class StateMachine:
                 known_jobs,
             )
 
-        self._log_status()
-
     @property
     def outputs(self) -> Optional[Env.Bindings[Value.Base]]:
         """
@@ -252,7 +249,6 @@ class StateMachine:
                     set(itertools.chain(*(self.jobs[j].dependencies for j in self.waiting)))
                     - self.finished
                 )
-                self._log_status()
                 return None
             job_id = runnable.pop()
             job = self.jobs[job_id]
@@ -407,27 +403,10 @@ class StateMachine:
     def logger(self) -> logging.Logger:
         if not self._logger:
             self._logger = logging.getLogger(self.logger_id)
-            # TODO: if we were truly unpickling in a new process, we'd need to reinitialize the
-            # logger object like this --
-            #
-            # if self.run_dir and not self._logger.hasHandlers():
-            #    fh = logging.FileHandler(os.path.join(self.run_dir, "workflow.log"))
-            #    fh.setFormatter(logging.Formatter(LOGGING_FORMAT))
-            #    self._logger.addHandler(fh)
-            #
-            # and close it later
+            # TODO: if we were truly unpickling in a new process, we'd need to add a new
+            # LoggingFileHandler to self._logger
         return self._logger
 
-    def _log_status(self) -> None:
-        self.logger.notice(  # pyre-fixme
-            _(
-                "workflow steps",
-                waiting=len(self.waiting),
-                outstanding=len(self.running),
-                finished=len(self.finished),
-            )
-        )
-
     def __getstate__(self) -> Dict[str, Any]:
         ans = dict(self.__dict__)
         del ans["_logger"]  # for Python pre-3.7 loggers: https://bugs.python.org/issue30520
@@ -461,7 +440,7 @@ def _scatter(
         if v.value:
             # condition is satisfied, so we'll "scatter" over a length-1 array
             array = [None]
-    digits = math.ceil(math.log10(len(array) + 1))
+    digits = math.ceil(math.log10(len(array))) if len(array) > 1 else 1
 
     # for each array element, schedule an instance of the body subgraph
     last_scatter_indices = None
@@ -472,9 +451,9 @@ def _scatter(
         scatter_stack_i = scatter_stack
         if isinstance(array_i, Value.Base):
             assert isinstance(section, Tree.Scatter)
-            scatter_stack_i = scatter_stack_i + [
-                (str(i).zfill(digits), Env.Binding(section.variable, array_i))
-            ]
+            str_i = str(i).zfill(digits)
+            assert len(str_i) <= digits
+            scatter_stack_i = scatter_stack_i + [(str_i, Env.Binding(section.variable, array_i))]
         scatter_indices_i = [p[0] for p in scatter_stack_i]
         assert last_scatter_indices is None or last_scatter_indices < scatter_indices_i
         last_scatter_indices = scatter_indices_i
@@ -629,15 +608,22 @@ def run_local_workflow(
     # provision run directory and log file
     run_id = run_id or workflow.name
     _run_id_stack = _run_id_stack or []
-    run_dir = provision_run_dir(workflow.name, run_dir)
+    run_dir = provision_run_dir(workflow.name, run_dir, last_link=not _run_id_stack)
 
     logger_prefix = logger_prefix or ["wdl"]
     logger_id = logger_prefix + ["w:" + run_id]
     logger = logging.getLogger(".".join(logger_id))
     logfile = os.path.join(run_dir, "workflow.log")
     with ExitStack() as cleanup:
-        fh = cleanup.enter_context(LoggingFileHandler(logger, logfile))  # pylint: disable=no-member
-        fh.setFormatter(logging.Formatter(LOGGING_FORMAT))
+        fh = cleanup.enter_context(
+            LoggingFileHandler(
+                logger,
+                logfile,
+                json=(
+                    cfg["logging"].get_bool("json") if cfg.has_option("logging", "json") else False
+                ),
+            )
+        )
         logger.notice(  # pyre-fixme
             _(
                 "workflow start",
@@ -651,9 +637,7 @@ def run_local_workflow(
         logger.debug(_("thread", ident=threading.get_ident()))
         write_values_json(inputs, os.path.join(run_dir, "inputs.json"), namespace=workflow.name)
 
-        terminating = cleanup.enter_context(  # pylint: disable=no-member
-            TerminationSignalFlag(logger)
-        )
+        terminating = cleanup.enter_context(TerminationSignalFlag(logger))
 
         # if we're the top-level workflow, provision CallCache and thread pools
         if not _run_id_stack:
@@ -662,9 +646,9 @@ def run_local_workflow(
             except importlib_metadata.PackageNotFoundError:
                 version = "UNKNOWN"
             logger.notice(_("miniwdl", version=version))  # pyre-fixme
-            assert not _thread_pools and not _cache
+            assert not _thread_pools
 
-            cache = cleanup.enter_context(CallCache(cfg, logger))  # pylint: disable=no-member
+            cache = _cache or cleanup.enter_context(CallCache(cfg, logger))
             cache.flock(logfile, exclusive=True)  # flock top-level workflow.log
 
             # Provision separate thread pools for tasks and sub-workflows. With just one pool, it'd
@@ -676,13 +660,9 @@ def run_local_workflow(
                 cfg["scheduler"].get_int("call_concurrency") or multiprocessing.cpu_count()
             )
             task_pool = futures.ThreadPoolExecutor(max_workers=max_workers)
-            cleanup.callback(  # pylint: disable=no-member
-                futures.ThreadPoolExecutor.shutdown, task_pool
-            )
+            cleanup.callback(futures.ThreadPoolExecutor.shutdown, task_pool)
             subwf_pool = futures.ThreadPoolExecutor(max_workers=max_workers)
-            cleanup.callback(  # pylint: disable=no-member
-                futures.ThreadPoolExecutor.shutdown, subwf_pool
-            )
+            cleanup.callback(futures.ThreadPoolExecutor.shutdown, subwf_pool)
             thread_pools = (task_pool, subwf_pool)
         else:
             assert _thread_pools and _cache
@@ -705,6 +685,7 @@ def run_local_workflow(
                 _test_pickle,
             )
         except:
+            _statusbar.abort()
             if not _run_id_stack:
                 # if we're the top-level worfklow, signal abort to anything still running
                 # concurrently on the thread pools (SIGUSR1 will be picked up by
@@ -776,6 +757,7 @@ def _workflow_main_loop(
                     }
                     # submit to appropriate thread pool
                     if isinstance(next_call.callee, Tree.Task):
+                        _statusbar.task_backlogged()
                         future = thread_pools[0].submit(run_local_task, *sub_args, **sub_kwargs)
                     elif isinstance(next_call.callee, Tree.Workflow):
                         future = thread_pools[1].submit(
@@ -824,17 +806,17 @@ def _workflow_main_loop(
         except Exception as exn2:
             logger.debug(traceback.format_exc())
             logger.critical(_("failed to write error.json", dir=run_dir, message=str(exn2)))
-        if isinstance(exn, RunFailed):
-            logger.error(
-                _("call failure propagating", **{"from": getattr(exn, "run_id"), "dir": run_dir})
-            )
-        else:
+        if not isinstance(exn, RunFailed):
             info = {"error": exn.__class__.__name__, "dir": run_dir}
             if str(exn):
                 info["message"] = str(exn)
             if hasattr(exn, "job_id"):
                 info["node"] = getattr(exn, "job_id")
             logger.error(_(str(wrapper), dir=run_dir, **error_json(exn)))
+        elif not isinstance(exn.__cause__, Terminated):
+            logger.error(
+                _("call failure propagating", **{"from": getattr(exn, "run_id"), "dir": run_dir},)
+            )
         # Cancel all future tasks that havent started
         for key in call_futures:
             key.cancel()
