@@ -31,7 +31,7 @@ class TestTaskRunner(unittest.TestCase):
             assert len(doc.tasks[0].required_inputs.subtract(doc.tasks[0].available_inputs)) == 0
             if isinstance(inputs, dict):
                 inputs = WDL.values_from_json(inputs, doc.tasks[0].available_inputs, doc.tasks[0].required_inputs)
-            rundir, outputs = WDL.runtime.run_local_task(cfg, doc.tasks[0], (inputs or WDL.Env.Bindings()), run_dir=self._dir, **kwargs)
+            rundir, outputs = WDL.runtime.run_local_task(cfg, doc.tasks[0], (inputs or WDL.Env.Bindings()), run_dir=self._dir, logger_prefix=[self.id()], **kwargs)
             self._rundir = rundir
         except WDL.runtime.RunFailed as exn:
             if expected_exception:
@@ -148,10 +148,10 @@ class TestTaskRunner(unittest.TestCase):
         }
         """)
 
-        std_error_msgs = [record for record in capture.records if str(record.msg).startswith("2|")]
+        std_error_msgs = [record for record in capture.records if str(record.name).endswith("stderr")]
 
-        self.assertEqual(std_error_msgs.pop(0).msg, "2| Start logging")
-        self.assertEqual(std_error_msgs.pop().msg, "2| End logging")
+        self.assertEqual(std_error_msgs.pop(0).msg, "Start logging")
+        self.assertEqual(std_error_msgs.pop().msg, "End logging")
         for record in std_error_msgs:
             line_written = int(record.msg.split('=')[1])
             self.assertGreater(record.created, line_written)
@@ -181,10 +181,10 @@ class TestTaskRunner(unittest.TestCase):
 
                 }
                 """)
-        std_error_msgs = [record for record in capture.records if str(record.msg).startswith("2|")]
+        std_error_msgs = [record for record in capture.records if str(record.name).endswith("stderr")]
 
         self.assertEqual(len(std_error_msgs), 6)
-        self.assertEqual(std_error_msgs[0].msg, "2| Part onePart two")
+        self.assertEqual(std_error_msgs[0].msg, "Part onePart two")
 
     def test_hello_blank(self):
         self._test_task(R"""
@@ -669,18 +669,22 @@ class TestTaskRunner(unittest.TestCase):
         self.assertEqual(self._test_task(txt, {"x": 22, "y": 99})["yy"], 99)
 
     def test_signal(self):
-        signal.alarm(3)
-        t0 = time.time()
-        self._test_task(R"""
-        version 1.0
-        task t {
-            command {
-                sleep 30
+        # also just covering codepaths for stderr status bar logging:
+        with WDL._util.configure_logger(force_tty=True) as set_status, WDL.runtime._statusbar.enable(set_status):
+            signal.alarm(10)
+            t0 = time.time()
+            self._test_task(R"""
+            version 1.0
+            task t {
+                command {
+                    sleep 30
+                }
             }
-        }
-        """, expected_exception=WDL.runtime.Terminated)
-        t1 = time.time()
-        self.assertLess(t1 - t0, 15)
+            """, expected_exception=WDL.runtime.Terminated)
+            t1 = time.time()
+            self.assertLess(t1 - t0, 20)
+        with WDL._util.configure_logger(json=True) as _:
+            pass
 
     def test_orphan_background_process(self):
         # TODO: https://github.com/chanzuckerberg/miniwdl/issues/211
@@ -774,6 +778,32 @@ class TestTaskRunner(unittest.TestCase):
         self._test_task(txt, {"memory": "1Gaga"}, expected_exception=WDL.Error.EvalError)
         self._test_task(txt, {"memory": "bogus"}, expected_exception=WDL.Error.EvalError)
 
+    def test_runtime_memory_limit(self):
+        txt = R"""
+        version 1.0
+        task limit {
+            input {
+                String memory
+            }
+            command <<<
+                cat /sys/fs/cgroup/memory/memory.limit_in_bytes
+            >>>
+            output {
+                Int memory_limit_in_bytes = read_int(stdout())
+            }
+            runtime {
+                cpu: 1
+                memory: "~{memory}"
+            }
+        }
+        """
+        cfg = WDL.runtime.config.Loader(logging.getLogger(self.id()), [])
+        outputs = self._test_task(txt, {"memory": "256MB"}, cfg=cfg)
+        self.assertGreater(outputs["memory_limit_in_bytes"], 300*1024*1024)
+        cfg.override({"task_runtime": {"memory_limit_multiplier": 0.9}})
+        outputs = self._test_task(txt, {"memory": "256MB"}, cfg=cfg)
+        self.assertLess(outputs["memory_limit_in_bytes"], 300*1024*1024)
+
     def test_input_files_rw(self):
         txt = R"""
         version 1.0
@@ -945,7 +975,7 @@ class TestTaskRunner(unittest.TestCase):
             outfile.write("Ben\n")
         inputs = {"files": [os.path.join(self._dir, "alyssa.txt"), os.path.join(self._dir, "ben.txt")]}
         cfg = WDL.runtime.config.Loader(logging.getLogger(self.id()), [])
-        cfg.override({"file_io": {"output_hardlinks": True}, "task_runtime": {"delete_work": "success"}})
+        cfg.override({"file_io": {"output_hardlinks": True, "delete_work": "success"}})
         output = self._test_task(txt, inputs, cfg=cfg)
         self.assertFalse(os.path.isdir(os.path.join(self._rundir, "work")))
         self.assertFalse(os.path.isdir(os.path.join(self._rundir, "write_")))
@@ -953,7 +983,7 @@ class TestTaskRunner(unittest.TestCase):
             self.assertTrue(os.path.isfile(fn) and not os.path.islink(fn))
 
         cfg = WDL.runtime.config.Loader(logging.getLogger(self.id()), [])
-        cfg.override({"file_io": {"output_hardlinks": False}, "task_runtime": {"delete_work": "success"}})
+        cfg.override({"file_io": {"output_hardlinks": False, "delete_work": "success"}})
         output = self._test_task(txt, inputs, cfg=cfg)
         self.assertTrue(os.path.isfile(os.path.join(self._rundir, "work", "foo.txt")))
 
